

def get_days_in_month(month: int, year: int):
    """Return the number of days in a given month and year."""
    if month == 2:
        # Check for leap years
        if year % 4 == 0 and (year % 100 != 0 or year % 400 == 0):
            return 29
        else:
            return 28
    elif month in {4, 6, 9, 11}:
        return 30
    else:
        return 31



import os
import asyncio
from fastapi import FastAPI
from tortoise.contrib.fastapi import register_tortoise
import yaml

from estimenergy.routers import energy_data, collector
from estimenergy.models import Collector
from estimenergy.common import instrumentator, settings


app = FastAPI(
    title="EstimEnergy",
)

register_tortoise(
    app,
    db_url=f"sqlite://{settings.db_path}",
    modules={"models": ["estimenergy.models"]},
    generate_schemas=True,
    add_exception_handlers=True,
)

instrumentator.instrument(app)
instrumentator.expose(app, include_in_schema=True)

app.include_router(energy_data.router)
app.include_router(collector.router)

@app.on_event("startup")
async def start_energy_collectors():
    
    with open(settings.config_path) as f:
        config_dict = yaml.safe_load(f)

    collector_names = [collector_dict["name"] for collector_dict in config_dict["collectors"]]
    collectors = await Collector.all()
    
    for collector in collectors:
        if collector.name not in collector_names:
            await collector.delete()
    

    for collector_dict in config_dict["collectors"]:
        collector = await Collector.filter(name=collector_dict["name"]).first()
        if collector is None:
            collector = Collector(**collector_dict)
            await collector.save()
    

    collectors = await Collector.all()

    for collector in collectors:
        asyncio.create_task(collector.connect())




SENSOR_TYPE_JSON = "json"
SENSOR_TYPE_FRIENDLY_NAME = "friendly_name"
SENSOR_TYPE_UNIQUE_ID = "unique_id"

JSON_NAME = "name"
JSON_HOST = "host"
JSON_PORT = "port"
JSON_COST_PER_KWH = "cost_per_kwh"
JSON_BASE_COST_PER_MONTH = "base_cost_per_month"
JSON_PAYMENT_PER_MONTH = "payment_per_month"
JSON_MIN_HOUR = "min_hour"
JSON_MAX_HOUR = "max_hour"
JSON_MAX_INCOMPLETE_DAYS = "max_incomplete_days"
JSON_BILLING_MONTH = "billing_month"
JSON_DATA = "data"

JSON_DAY_KWH = "day_kwh"
FRIENDLY_DAY_KWH = "Daily Energy Usage"
UNIQUE_ID_DAY_KWH = "day_kwh"

JSON_DAY_COST = "day_cost"
FRIENDLY_DAY_COST = "Daily Energy Cost"
UNIQUE_ID_DAY_COST = "day_cost"

JSON_DAY_COST_DIFFERENCE = "day_cost_difference"
FRIENDLY_DAY_COST_DIFFERENCE = "Daily Energy Cost Difference"
UNIQUE_ID_DAY_COST_DIFFERENCE = "day_cost_difference"

JSON_MONTH_KWH_RAW = "predicted_month_kwh_raw"
FRIENDLY_MONTH_KWH_RAW = "Predicted Monthly Energy Usage Raw"
UNIQUE_ID_MONTH_KWH_RAW = "predicted_month_kwh_raw"

JSON_MONTH_COST_RAW = "predicted_month_cost_raw"
FRIENDLY_MONTH_COST_RAW = "Predicted Monthly Energy Cost Raw"
UNIQUE_ID_MONTH_COST_RAW = "predicted_month_cost_raw"

JSON_MONTH_COST_DIFFERENCE_RAW = "predicted_month_cost_difference_raw"
FRIENDLY_MONTH_COST_DIFFERENCE_RAW = "Predicted Monthly Energy Cost Difference Raw"
UNIQUE_ID_MONTH_COST_DIFFERENCE_RAW = "predicted_month_cost_difference_raw"

JSON_YEAR_KWH_RAW = "predicted_year_kwh_raw"
FRIENDLY_YEAR_KWH_RAW = "Predicted Yearly Energy Usage Raw"
UNIQUE_ID_YEAR_KWH_RAW = "predicted_year_kwh_raw"

JSON_YEAR_COST_RAW = "predicted_year_cost_raw"
FRIENDLY_YEAR_COST_RAW = "Predicted Yearly Energy Cost Raw"
UNIQUE_ID_YEAR_COST_RAW = "predicted_year_cost_raw"

JSON_YEAR_COST_DIFFERENCE_RAW = "predicted_year_cost_difference_raw"
FRIENDLY_YEAR_COST_DIFFERENCE_RAW = "Predicted Yearly Energy Cost Difference Raw"
UNIQUE_ID_YEAR_COST_DIFFERENCE_RAW = "predicted_year_cost_difference_raw"

JSON_MONTH_KWH = "predicted_month_kwh"
FRIENDLY_MONTH_KWH = "Predicted Monthly Energy Usage"
UNIQUE_ID_MONTH_KWH = "predicted_month_kwh"

JSON_MONTH_COST = "predicted_month_cost"
FRIENDLY_MONTH_COST = "Predicted Monthly Energy Cost"
UNIQUE_ID_MONTH_COST = "predicted_month_cost"

JSON_MONTH_COST_DIFFERENCE = "predicted_month_cost_difference"
FRIENDLY_MONTH_COST_DIFFERENCE = "Predicted Monthly Energy Cost Difference"
UNIQUE_ID_MONTH_COST_DIFFERENCE = "predicted_month_cost_difference"

JSON_YEAR_KWH = "predicted_year_kwh"
FRIENDLY_YEAR_KWH = "Predicted Yearly Energy Usage"
UNIQUE_ID_YEAR_KWH = "predicted_year_kwh"

JSON_YEAR_COST = "predicted_year_cost"
FRIENDLY_YEAR_COST = "Predicted Yearly Energy Cost"
UNIQUE_ID_YEAR_COST = "predicted_year_cost"

JSON_YEAR_COST_DIFFFERENCE = "predicted_year_cost_difference"
FRIENDLY_YEAR_COST_DIFFFERENCE = "Predicted Yearly Energy Cost Difference"
UNIQUE_ID_YEAR_COST_DIFFFERENCE = "predicted_year_cost_difference"

SENSOR_TYPES = [
    {
        SENSOR_TYPE_JSON: JSON_DAY_KWH,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_DAY_KWH,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_DAY_KWH,
    },
    {
        SENSOR_TYPE_JSON: JSON_DAY_COST,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_DAY_COST,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_DAY_COST,
    },
    {
        SENSOR_TYPE_JSON: JSON_DAY_COST_DIFFERENCE,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_DAY_COST_DIFFERENCE,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_DAY_COST_DIFFERENCE,
    },
    {
        SENSOR_TYPE_JSON: JSON_MONTH_KWH_RAW,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_MONTH_KWH_RAW,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_MONTH_KWH_RAW,
    },
    {
        SENSOR_TYPE_JSON: JSON_MONTH_COST_RAW,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_MONTH_COST_RAW,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_MONTH_COST_RAW,
    },
    {
        SENSOR_TYPE_JSON: JSON_MONTH_COST_DIFFERENCE_RAW,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_MONTH_COST_DIFFERENCE_RAW,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_MONTH_COST_DIFFERENCE_RAW,
    },
    {
        SENSOR_TYPE_JSON: JSON_YEAR_KWH_RAW,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_YEAR_KWH_RAW,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_YEAR_KWH_RAW,
    },
    {
        SENSOR_TYPE_JSON: JSON_YEAR_COST_RAW,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_YEAR_COST_RAW,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_YEAR_COST_RAW,
    },
    {
        SENSOR_TYPE_JSON: JSON_YEAR_COST_DIFFERENCE_RAW,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_YEAR_COST_DIFFERENCE_RAW,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_YEAR_COST_DIFFERENCE_RAW,
    },
    {
        SENSOR_TYPE_JSON: JSON_MONTH_KWH,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_MONTH_KWH,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_MONTH_KWH,
    },
    {
        SENSOR_TYPE_JSON: JSON_MONTH_COST,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_MONTH_COST,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_MONTH_COST,
    },
    {
        SENSOR_TYPE_JSON: JSON_MONTH_COST_DIFFERENCE,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_MONTH_COST_DIFFERENCE,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_MONTH_COST_DIFFERENCE,
    },
    {
        SENSOR_TYPE_JSON: JSON_YEAR_KWH,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_YEAR_KWH,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_YEAR_KWH,
    },
    {
        SENSOR_TYPE_JSON: JSON_YEAR_COST,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_YEAR_COST,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_YEAR_COST,
    },
    {
        SENSOR_TYPE_JSON: JSON_YEAR_COST_DIFFFERENCE,
        SENSOR_TYPE_FRIENDLY_NAME: FRIENDLY_YEAR_COST_DIFFFERENCE,
        SENSOR_TYPE_UNIQUE_ID: UNIQUE_ID_YEAR_COST_DIFFFERENCE,
    },
]





import os
from setuptools import setup

version = os.environ.get("RELEASE_VERSION", None)

setup(
    name='estimenergy',
    version=version,
    description='EstimEnergy Python Package',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/EuleMitKeule/EstimEnergy',
    author='Lennard Beers',
    author_email='l.beers@outlook.de',
    keywords='estimenergy api client',
    packages=['estimenergy'],
    python_requires='>=3.6',
    install_requires=[
        "requests",
        "uvicorn[standard]",
        "fastapi",
        "fastapi-crudrouter",
        "httpx",
        "aioesphomeapi",
        "tortoise_orm",
        "prometheus_client",
        "prometheus-fastapi-instrumentator",
        "python-dotenv",
    ],
)